;;; dialog-y-n.el v1.2

;;; based on map-ynp.el by Roland McGrath <roland@gnu.ai.mit.edu>
;; Keywords: lisp, extensions

;; Modified by Alon Albert <alon@milcse.rtsg.ai.edu> to pop a dialog box

;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2, or (at your option)
;;; any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;

;;; Commentary:

;;; Modified map-y-or-n-p and yes-or-no-p to pop up a dialog box

;;; History:

;;; v1.2 July 7 1993: <Alon Albert>
;;;      - User defined function bug fixed
;;; v1.1 July 7 1993: <Alon Albert>
;;;    - Removed keymap stuff (irrelevent)
;;;    - map-y-n-p menu displays a full description of action using optional
;;;      HELP  variable.
;;; v1.0 July 6 1993: <Alon Albert>
;;;    - First release.
;;;    - Based on map-y-or-n-p by Roland McGrath <roland@gnu.ai.mit.edu>

;;; Instalation:
;;;   put this file in your load-path and insert the following in .emacs
;;;
;;;   (if (string= window-system "x")
;;;       (load "dialog-y-n"))

;; Einf"ugung: Werner Fink
;;(defun dl-mouse-position ()
;;   (if (boundp 'mouse-init-event)
;;      (mouse-position)
;;    (set-mouse-position last-event-frame
;;                        (/ (frame-width) 2)
;;                        (eval '(. (/ (frame-height) 2))))
;;    (unfocus-frame)
;;    (cons last-event-frame
;;       (cons (/ (frame-width) 2) (eval '(. (/ (frame-height) 2)))))))
;;
;;(fset 'dl-mouse-set-point (symbol-function 'mouse-set-point))
;;
;;(defun mouse-set-point (click)
;;  (interactive "e")
;;  (dl-mouse-set-point click)
;;  (if (boundp 'mouse-init-event) nil (setq mouse-init-event t)))
;; Ende

(defvar dialog-last-answer 1
  "Last answer from yes-or-no-p: 1 if yes, 0 if no")

(defun map-y-or-n-p (prompter actor list &optional help action-alist
   no-cursor-in-echo-area)
  "Ask a series of boolean questions.
Takes args PROMPTER ACTOR LIST, and optional args HELP and ACTION-ALIST.

LIST is a list of objects, or a function of no arguments to return the next
object or nil.

If PROMPTER is a string, the prompt is \(format PROMPTER OBJECT\).  If not
a string, PROMPTER is a function of one arg (an object from LIST), which
returns a string to be used as the prompt for that object.  If the return
value is not a string, it is eval'd to get the answer; it may be nil to
ignore the object, t to act on the object without asking the user, or a
form to do a more complex prompt.

ACTOR is a function of one arg (an object from LIST),
which gets called with each object that the user answers `yes' for.

If HELP is given, it is a list (OBJECT OBJECTS ACTION),
where OBJECT is a string giving the singular noun for an elt of LIST;
OBJECTS is the plural noun for elts of LIST, and ACTION is a transitive
verb describing ACTOR.  The default is \(\"object\" \"objects\" \"act on\"\).

A popup menu is displayed and the user chooses his pick.

If ACTION-ALIST is given, it is an alist (KEY FUNCTION HELP) of extra keys
that will be accepted.  KEY is a character; FUNCTION is a function of one
arg (an object from LIST); HELP is a string.  When the user hits KEY,
FUNCTION is called.  If it returns non-nil, the object is considered
\"acted upon\", and the next object from LIST is processed.  If it returns
nil, the prompt is repeated for the same object.

This function uses `query-replace-map' to define the standard responses,
but not all of the responses which `query-replace' understands
are meaningful here.

Returns the number of actions taken."
  (let* ((object (if help (nth 0 help) "object"))
         (objects (if help (nth 1 help) "objects"))
         (action (if help (nth 2 help) "act on"))
         ;; Make a menu
         (popup-menu (append (list (cons (format "    %s %s"
                                                 action
                                                 object) 'act))
                             (list (cons (format "    skip %s"
                                                 object) 'skip))
                             (list (cons (format "    %s all remaining %s"
                                                 action
                                                 objects) 'automatic))
                             (list (cons (format "    %s current %s and quit"
                                                 action
                                                 object) 'act-and-exit))
                             (list (cons (format "    Quit") 'exit))
                             (mapcar (function
                                      (lambda (elt)
                                        (cons (concat "    " (nth 2 elt))
                                              (vector (nth 1 elt)))))
                                      action-alist)))
         (x (/ (frame-pixel-width)  3))
         (y (/ (frame-pixel-height) 3))
         (mouse-pos (mouse-pixel-position))
         (mouse-pos (if (nth 1 mouse-pos)
                        mouse-pos
                      (frame-root-window last-event-frame)
                      (set-mouse-pixel-position (car mouse-pos) x y)
                      (unfocus-frame)
                      (append (list (car mouse-pos))
			(append (list x) y))))
         (x (car (cdr mouse-pos)))
         (y (cdr (cdr mouse-pos)))
         (pos (list (list x (+ y 1)) (car mouse-pos)))
         (actions 0)
         prompt char elt tail def
         (next (if (or (symbolp list)
                       (subrp list)
                       (byte-code-function-p list)
                       (and (consp list)
                            (eq (car list) 'lambda)))
                   (function (lambda ()
                               (setq elt (funcall list))))
                 (function (lambda ()
                             (if list
                                 (progn
                                   (setq elt (car list)
                                         list (cdr list))
                                   t)
                               nil))))))

    (if (stringp prompter)
        (setq prompter (` (lambda (object)
                            (format (, prompter) object)))))
    (while (funcall next)
      (setq prompt (funcall prompter elt))
      (if (stringp prompt)
          (progn
            (setq quit-flag nil)
            ;; Prompt the user about this object.
            (let* ((cursor-in-echo-area t)
;;;;              (popup-menu (list "dialog-y-n menu"
;;;;                                (append (list prompt) popup-menu))))
;;;;                                (append (list prompt prompt nil) popup-menu))))
                  (popup-menu (list prompt (append (list prompt) popup-menu))))
              (setq def (x-popup-menu pos popup-menu)))
            (cond ((eq def 'exit)
                   (setq next (function (lambda () nil))))
                  ((eq def 'act)
                   ;; Act on the object.
                   (funcall actor elt)
                   (setq actions (+ actions 1)))
                  ((eq def 'skip)
                   ;; Skip the object.
                   )
                  ((eq def 'act-and-exit)
                   ;; Act on the object and then exit.
                   (funcall actor elt)
                   (setq actions (+ actions 1)
                         next (function (lambda () nil))))
                  ((eq def 'quit) 
                   (setq quit-flag t)
                   (setq next (` (lambda ()
                                   (setq next '(, next))
                                   '(, elt)))))
                  ((eq def 'automatic)
                   ;; Act on this and all following objects.
                   (if (eval (funcall prompter elt))
                       (progn
                         (funcall actor elt)
                         (setq actions (+ actions 1))))
                   (while (funcall next)
                     (if (eval (funcall prompter elt))
                         (progn
                           (funcall actor elt)
                           (setq actions (+ actions 1))))))
                  ((vectorp def)
                   ;; A user-defined function.
                   (if (funcall (aref def 0) elt) ;Call its function.
                       ;; The function has eaten this object.
                       (setq actions (+ actions 1))
                     ;; Regurgitated; try again.
                     (setq next (` (lambda ()
                                     (setq next '(, next))
                                     '(, elt))))))))
        (if (eval prompt)
            (progn
              (funcall actor elt)
              (setq actions (1+ actions))))))
    ;; Clear the last prompt from the minibuffer.
    (message "")
    ;; Return the number of actions that were taken.
    actions))


(defun yes-or-no-p (prompt)
  "Ask user a yes-or-no question in a popup window. Return t if answer is yes.
Takes one argument, which is the string to display to ask the question."
  (let* ((popup-menu
;;;; (list "dialog-yes-no menu"
     (list prompt
                           (list prompt
                                 (cons "        Yes" t)
                                 (cons "        No" nil)
			         )))
;;;;			         prompt nil )))
         (x (/ (frame-pixel-width)  3))
         (y (/ (frame-pixel-height) 3))
         (mouse-pos (mouse-pixel-position))
         (mouse-pos (if (nth 1 mouse-pos)
                        mouse-pos
                      (frame-root-window last-event-frame)
                      (set-mouse-pixel-position (car mouse-pos) x y)
                      (unfocus-frame)
                      (append (list (car mouse-pos))
                        (append (list x) y))))
         (x (car (cdr mouse-pos)))
         (y (cdr (cdr mouse-pos)))
         (pos (list (list x (+ y dialog-last-answer)) (car mouse-pos)))
         (answer (x-popup-menu pos popup-menu)))
    (setq dialog-last-answer (if answer 1 0))
    answer))
         
(provide 'dialog-y-n)

